const Gtk = imports.gi.Gtk;
const St = imports.gi.St;
const Main = imports.ui.main;
const IconGrid = imports.ui.iconGrid;
const Lang = imports.lang;
const Dash = imports.ui.dash;

const Extension = imports.misc.extensionUtils.getCurrentExtension();
const NotesView = Extension.imports.view.NotesView;

let dash, viewSelector;
let _a11yFocusPageOld;

let showNotesIcon;
let notesPage;
let signal_notes, signal_apps, signal_show, signal_hide;

/* The button to be added to the dash. */
const ShowNotesIcon = new Lang.Class({
    Name: 'ShowNotesIcon',
    Extends: Dash.DashItemContainer,

    _init: function() {
        this.parent();

        this.toggleButton = new St.Button({ style_class: 'show-apps',
                                            track_hover: true,
                                            can_focus: true,
                                            toggle_mode: true });
        this._iconActor = null;
        this.icon = new IconGrid.BaseIcon(_("Show Notes"),
                                           { setSizeManually: true,
                                             showLabel: false,
                                             createIcon: Lang.bind(
                                                this, this._createIcon) });
        this.toggleButton.add_actor(this.icon.actor);
        this.toggleButton._delegate = this;

        this.setChild(this.toggleButton);
    },

    _createIcon: function(size) {
        this._iconActor = new St.Icon({ icon_name: 'notes-icon',
                                        icon_size: size,
                                        style_class: 'show-apps-icon',
                                        track_hover: true });
        return this._iconActor;
    }
});

/* Replace ViewSelector's method to allow notes view to be displayed. */
function onButtonToggled() {
    if (viewSelector.active)
        viewSelector.reset();
    else if (dash.showAppsButton.checked)
        viewSelector._showPage(viewSelector._appsPage);
    else if (showNotesIcon.toggleButton.checked)
        viewSelector._showPage(notesPage);
    else
        viewSelector._showPage(viewSelector._workspacesPage);
}

/* Replace ViewSelector's to allow notes view to be selected with CtrlAltTab. */
function _a11yFocusPage(page) {
    this._showAppsButton.checked = page == this._appsPage;
    showNotesIcon.toggleButton.checked = page == notesPage;
    page.navigate_focus(null, Gtk.DirectionType.TAB_FORWARD, false);
}

function init() {
    dash = Main.overview._dash;
    viewSelector = Main.overview._viewSelector;

    _a11yFocusPageOld = Main.overview._viewSelector._a11yFocusPage;
}

function enable() {
    // Replace show apps button
    showNotesIcon = new ShowNotesIcon();
    showNotesIcon.icon.setIconSize(dash.iconSize);
    dash._hookUpLabel(showNotesIcon);

    dash._container.remove_actor(dash._showAppsIcon.actor);
    dash._container.add_actor(showNotesIcon.actor);

    // Add notes page
    notesPage = viewSelector._addPage(new NotesView().actor, null, _("Notes"),
                                      'system-run-symbolic');
    notesPage.hide();

    // Rewire viewSelector's behaviour
    signal_notes = showNotesIcon.toggleButton.connect('notify::checked',
                                                      onButtonToggled);
    // TODO Need to disconnect old signal, but there is no mechanism to do so.
    signal_apps = viewSelector._showAppsButton.connect('notify::checked',
                                                       onButtonToggled);
    signal_show = Main.overview.connect('showing',
        function () { showNotesIcon.toggleButton.checked = false; });
    signal_hide = Main.overview.connect('hiding',
        function () { showNotesIcon.toggleButton.checked = false; });
    Main.overview._viewSelector._a11yFocusPage = _a11yFocusPage;
}

function disable() {
    viewSelector._a11yFocusPage = _a11yFocusPageOld;
    Main.overview.disconnect(signal_hide);
    Main.overview.disconnect(signal_show);
    viewSelector._showAppsButton.disconnect(signal_apps);
    showNotesIcon.toggleButton.disconnect(signal_notes);

    notesPage.destroy();
    notesPage = null;

    dash._container.remove_actor(showNotesIcon.actor);
    dash._container.add_actor(dash._showAppsIcon.actor);
    showNotesIcon.destroy();
    showNotesIcon = null;
}
