// -*- mode: js; js-indent-level: 4; indent-tabs-mode: nil -*-

const GLib = imports.gi.GLib;
const Gtk = imports.gi.Gtk;
const Lang = imports.lang;
const St = imports.gi.St;

const IconGrid = imports.ui.iconGrid;
const Main = imports.ui.main;
const Overview = imports.ui.overview;

const PATH = "/home/s/.local/share/gnome-shell/.notes"

/* This is the page object to create our notes view. */
const NotesView = new Lang.Class({
    Name: 'NotesView',

    _init: function() {
        this.notes = new Array();

        // Use IconGrid from AppsDisplay to arrange the notes.
        //IconGrid.IconGrid.prototype['_onStyleChanged'] = _onStyleChanged;
        this._grid = new IconGrid.IconGrid({ xAlign: St.Align.START });
        this._grid.actor.connect('style-changed',
                                 Lang.bind(this._grid, _onStyleChanged));

        this.actor = new St.ScrollView({ x_fill: true,
                                         y_fill: false,
                                         y_align: St.Align.START,
                                         style_class: 'vfade' });
        this.actor.add_actor(this._grid.actor);
        this.actor.set_policy(Gtk.PolicyType.NEVER, Gtk.PolicyType.AUTOMATIC);
        this.actor.connect('notify::mapped', Lang.bind(this,
            function() {
                if (!this.actor.mapped)
                    return;

                let adjustment = this.actor.vscroll.adjustment;
                let direction = Overview.SwipeScrollDirection.VERTICAL;
                Main.overview.setScrollAdjustment(adjustment, direction);

                // Reset scroll on mapping
                adjustment.value = 0;
            }));

        this._workId = Main.initializeDeferredWork(
            this.actor, Lang.bind(this, this._redisplay));
    },

    _redisplay: function() {
        // Read contents from file
        let content;
        try {
            content = GLib.file_get_contents(PATH)[1];
        } catch (e) {
            content = "";
        }

        // Split individual notes using a private unicode character.
        content = content.toString().split("\uE000");

        let note;
        for (let i=0; i < content.length; i++) {
            note = new NoteIcon(this, content[i]);
            this.notes.push(note);
            this._grid.addItem(note.actor);
        }
        if (content[content.length-1].length > 0) {
            note = new NoteIcon(this, "");
            this.notes.push(note);
            this._grid.addItem(note.actor);
        }
    },

    // Save notes to file.
    save: function() {
        let content = "";
        for (let i=0; i < this.notes.length; i++) {
            content = content.concat("\uE000", this.notes[i].actor.get_text());
        }

        let byteData = imports.byteArray.fromString(content.slice(1), "UTF-8");
        GLib.file_set_contents(PATH, byteData);

        // Add empty note
        if (this.notes[this.notes.length-1].actor.get_text().length > 0) {
            let note = new NoteIcon(this, "");
            this.notes.push(note);
            this._grid.addItem(note.actor);
        }
    }
});

/* Increase size of IconGrid items to allow full sized notes. */
function _onStyleChanged() {
    let themeNode = this.actor.get_theme_node();
    this._spacing = themeNode.get_length('spacing');
    this._hItemSize = 200;
    this._vItemSize = 280;
    this._grid.queue_relayout();
}

/* The note item. */
const NoteIcon = new Lang.Class({
    Name: 'NoteIcon',

    _init : function(notesView, text) {
        this.notesView = notesView;

        this.actor = new St.Entry({ style_class: 'note' });
        this.actor._delegate = this;
        this.actor.natural_width = 200;
        this.actor.natural_height = 280;
        this.actor.set_text(text);

        let clutterText = this.actor.get_clutter_text();
        clutterText.set_single_line_mode(false);
        clutterText.set_activatable(false);
        clutterText.set_line_wrap(true);

        // Save data when focus leaves widget.
        clutterText.connect('key-focus-out', Lang.bind(this.notesView,
                                                       this.notesView.save));
    }
});
